cd %SRC_DIR%\build
set DESTDIR=C:\clangb
:: Drop drive letter and colon
SET _PREFIX=%PREFIX:~2%
ninja install
echo LIZZY1_1 %DESTDIR%%_PREFIX%\Library
echo LIZZY2_1 %LIBRARY_BIN%
pushd %DESTDIR%%_PREFIX%\Library
  copy bin\libclang.dll %LIBRARY_PREFIX%\bin\
popd
